@@ -1,23 +1,22 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Authors: Bertrand Guiheneuf <bertrand@helixcode.com>
- *          Michael Zucchi <notzed@ximian.com>
- *          Jeffrey Stedfast <fejj@ximian.com>
- *
  * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
- * This library is free software you can redistribute it and/or modify it
+ * This library is free software: you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published by
  * the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
  * for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this library. If not, see <http://www.gnu.org/licenses/>.
  *
+ * Authors: Bertrand Guiheneuf <bertrand@helixcode.com>
+ *          Michael Zucchi <notzed@ximian.com>
+ *          Jeffrey Stedfast <fejj@ximian.com>
  */
 
 #if !defined (__CAMEL_H_INSIDE__) && !defined (CAMEL_COMPILATION)
@@ -105,9 +104,14 @@ struct _CamelDataWrapperClass {
 						 GOutputStream *output_stream,
 						 GCancellable *cancellable,
 						 GError **error);
+	gboolean	(*construct_from_input_stream_sync)
+						(CamelDataWrapper *data_wrapper,
+						 GInputStream *input_stream,
+						 GCancellable *cancellable,
+						 GError **error);
 
 	/* Reserved slots. */
-	gpointer reserved[4];
+	gpointer reserved[3];
 };
 
 GType		camel_data_wrapper_get_type	(void);
@@ -208,6 +212,22 @@ gssize		camel_data_wrapper_decode_to_output_stream_finish
 						(CamelDataWrapper *data_wrapper,
 						 GAsyncResult *result,
 						 GError **error);
+gboolean	camel_data_wrapper_construct_from_input_stream_sync
+						(CamelDataWrapper *data_wrapper,
+						 GInputStream *input_stream,
+						 GCancellable *cancellable,
+						 GError **error);
+void		camel_data_wrapper_construct_from_input_stream
+						(CamelDataWrapper *data_wrapper,
+						 GInputStream *input_stream,
+						 gint io_priority,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	camel_data_wrapper_construct_from_input_stream_finish
+						(CamelDataWrapper *data_wrapper,
+						 GAsyncResult *result,
+						 GError **error);
 
 G_END_DECLS
 
